@@ -1,16 +1,22 @@
 /*
-Command mox is a modern full-featured open source secure mail server for
+Command mox is a modern, secure, full-featured, open source mail server for
 low-maintenance self-hosted email.
 
-  - Quick and easy to set up with quickstart and automatic TLS with ACME and
-    Let's Encrypt.
-  - IMAP4 with extensions for accessing email.
-  - SMTP with SPF, DKIM, DMARC, DNSBL, MTA-STS, TLSRPT for exchanging email.
-  - Reputation-based and content-based spam filtering.
-  - Internationalized email.
-  - Admin web interface.
+Mox is started with the "serve" subcommand, but mox also has many other
+subcommands.
 
-# Commands
+Many of those commands talk to a running mox instance, through the ctl file in
+the data directory. Specify the configuration file (that holds the path to the
+data directory) through the -config flag or MOXCONF environment variable.
+
+Commands that don't talk to a running mox instance are often for
+testing/debugging email functionality. For example for parsing an email message,
+or looking up SPF/DKIM/DMARC records.
+
+Below is the usage information as printed by the command when started without
+any parameters. Followed by the help and usage information for each command.
+
+# Usage
 
 	mox [-config config/mox.conf] [-pedantic] ...
 	mox serve
@@ -44,12 +50,17 @@ low-maintenance self-hosted email.
 	mox config domain rm domain
 	mox config describe-sendmail >/etc/moxsubmit.conf
 	mox config printservice >mox.service
+	mox config ensureacmehostprivatekeys
 	mox example [name]
 	mox checkupdate
 	mox cid cid
 	mox clientconfig domain
-	mox dkim gened25519 >$selector._domainkey.$domain.ed25519key.pkcs8.pem
-	mox dkim genrsa >$selector._domainkey.$domain.rsakey.pkcs8.pem
+	mox dane dial host:port
+	mox dane dialmx domain [destination-host]
+	mox dane makerecord usage selector matchtype [certificate.pem | publickey.pem | privatekey.pem]
+	mox dns lookup [ptr | mx | cname | ips | a | aaaa | ns | txt | srv | tlsa] name
+	mox dkim gened25519 >$selector._domainkey.$domain.ed25519.privatekey.pkcs8.pem
+	mox dkim genrsa >$selector._domainkey.$domain.rsa2048.privatekey.pkcs8.pem
 	mox dkim lookup selector domain
 	mox dkim txt <$selector._domainkey.$domain.key.pkcs8.pem
 	mox dkim verify message
@@ -79,10 +90,6 @@ low-maintenance self-hosted email.
 	mox message parse message.eml
 	mox reassignthreads [account]
 
-Many commands talk to a running mox instance, through the ctl file in the data
-directory. Specify the configuration file (that holds the path to the data
-directory) through the -config flag or MOXCONF environment variable.
-
 # mox serve
 
 Start mox, serving SMTP/IMAP/HTTPS.
@@ -92,6 +99,8 @@ IMAP. HTTP listeners are started for the admin/account web interfaces, and for
 automated TLS configuration. Missing essential TLS certificates are immediately
 requested, other TLS certificates are requested on demand.
 
+Only implemented on unix systems, not Windows.
+
 	usage: mox serve
 
 # mox quickstart
@@ -245,6 +254,12 @@ The message is printed to stdout and is in standard internet mail format.
 
 Import a maildir into an account.
 
+The mbox/maildir archive is accessed and imported by the running mox process, so
+it must have access to the archive files. The default suggested systemd service
+file isolates mox from most of the file system, with only the "data/" directory
+accessible, so you may want to put the mbox/maildir archive files in a
+directory like "data/import/" to make it available to mox.
+
 By default, messages will train the junk filter based on their flags and, if
 "automatic junk flags" configuration is set, based on mailbox naming.
 
@@ -258,9 +273,6 @@ uploading a zip or tgz file with mbox and/or maildirs.
 Mailbox flags, like "seen", "answered", will be imported. An optional
 dovecot-keywords file can specify additional flags, like Forwarded/Junk/NotJunk.
 
-The maildir files/directories are read by the mox process, so make sure it has
-access to the maildir directories/files.
-
 	usage: mox import maildir accountname mailboxname maildir
 
 # mox import mbox
@@ -269,6 +281,12 @@ Import an mbox into an account.
 
 Using mbox is not recommended, maildir is a better defined format.
 
+The mbox/maildir archive is accessed and imported by the running mox process, so
+it must have access to the archive files. The default suggested systemd service
+file isolates mox from most of the file system, with only the "data/" directory
+accessible, so you may want to put the mbox/maildir archive files in a
+directory like "data/import/" to make it available to mox.
+
 By default, messages will train the junk filter based on their flags and, if
 "automatic junk flags" configuration is set, based on mailbox naming.
 
@@ -279,9 +297,6 @@ recipients to be accepted, unless other reputation signals prevent that.
 Users can also import mailboxes/messages through the account web page by
 uploading a zip or tgz file with mbox and/or maildirs.
 
-The mailbox is read by the mox process, so make sure it has access to the
-maildir directories/files.
-
 	usage: mox import mbox accountname mailboxname mbox
 
 # mox export maildir
@@ -341,6 +356,8 @@ during those commands instead of during "data".
 	usage: mox localserve
 	  -dir string
 	    	configuration storage directory (default "$userconfigdir/mox-localserve")
+	  -initonly
+	    	write configuration files and exit
 	  -ip string
 	    	serve on this ip instead of default 127.0.0.1 and ::1. only used when writing configuration, at first launch.
 
@@ -541,6 +558,31 @@ date version.
 
 	usage: mox config printservice >mox.service
 
+# mox config ensureacmehostprivatekeys
+
+Ensure host private keys exist for TLS listeners with ACME.
+
+In mox.conf, each listener can have TLS configured. Long-lived private key files
+can be specified, which will be used when requesting ACME certificates.
+Configuring these private keys makes it feasible to publish DANE TLSA records
+for the corresponding public keys in DNS, protected with DNSSEC, allowing TLS
+certificate verification without depending on a list of Certificate Authorities
+(CAs). Previous versions of mox did not pre-generate private keys for use with
+ACME certificates, but would generate private keys on-demand. By explicitly
+configuring private keys, they will not change automatedly with new
+certificates, and the DNS TLSA records stay valid.
+
+This command looks for listeners in mox.conf with TLS with ACME configured. For
+each missing host private key (of type rsa-2048 and ecdsa-p256) a key is written
+to config/hostkeys/. If a certificate exists in the ACME "cache", its private
+key is copied. Otherwise a new private key is generated. Snippets for manually
+updating/editing mox.conf are printed.
+
+After running this command, and updating mox.conf, run "mox config dnsrecords"
+for a domain and create the TLSA DNS records it suggests to enable DANE.
+
+	usage: mox config ensureacmehostprivatekeys
+
 # mox example
 
 List available examples, or print a specific example.
@@ -553,7 +595,7 @@ Check if a newer version of mox is available.
 
 A single DNS TXT lookup to _updates.xmox.nl tells if a new version is
 available. If so, a changelog is fetched from https://updates.xmox.nl, and the
-individual entries validated with a builtin public key. The changelog is
+individual entries verified with a builtin public key. The changelog is
 printed.
 
 	usage: mox checkupdate
@@ -582,6 +624,80 @@ configured over otherwise secured connections, like a VPN.
 
 	usage: mox clientconfig domain
 
+# mox dane dial
+
+Dial the address using TLS with certificate verification using DANE.
+
+Data is copied between connection and stdin/stdout until either side closes the
+connection.
+
+	usage: mox dane dial host:port
+	  -usages string
+	    	allowed usages for dane, comma-separated list (default "pkix-ta,pkix-ee,dane-ta,dane-ee")
+
+# mox dane dialmx
+
+Connect to MX server for domain using STARTTLS verified with DANE.
+
+If no destination host is specified, regular delivery logic is used to find the
+hosts to attempt delivery too. This involves following CNAMEs for the domain,
+looking up MX records, and possibly falling back to the domain name itself as
+host.
+
+If a destination host is specified, that is the only candidate host considered
+for dialing.
+
+With a list of destinations gathered, each is dialed until a successful SMTP
+session verified with DANE has been initialized, including EHLO and STARTTLS
+commands.
+
+Once connected, data is copied between connection and stdin/stdout, until
+either side closes the connection.
+
+This command follows the same logic as delivery attempts made from the queue,
+sharing most of its code.
+
+	usage: mox dane dialmx domain [destination-host]
+	  -ehlohostname string
+	    	hostname to send in smtp ehlo command (default "localhost")
+
+# mox dane makerecord
+
+Print TLSA record for given certificate/key and parameters.
+
+Valid values:
+- usage: pkix-ta (0), pkix-ee (1), dane-ta (2), dane-ee (3)
+- selector: cert (0), spki (1)
+- matchtype: full (0), sha2-256 (1), sha2-512 (2)
+
+Common DANE TLSA record parameters are: dane-ee spki sha2-256, or 3 1 1,
+followed by a sha2-256 hash of the DER-encoded "SPKI" (subject public key info)
+from the certificate. An example DNS zone file entry:
+
+	_25._tcp.example.com. TLSA 3 1 1 133b919c9d65d8b1488157315327334ead8d83372db57465ecabf53ee5748aee
+
+The first usable information from the pem file is used to compose the TLSA
+record. In case of selector "cert", a certificate is required. Otherwise the
+"subject public key info" (spki) of the first certificate or public or private
+key (pkcs#8, pkcs#1 or ec private key) is used.
+
+	usage: mox dane makerecord usage selector matchtype [certificate.pem | publickey.pem | privatekey.pem]
+
+# mox dns lookup
+
+Lookup DNS name of given type.
+
+Lookup always prints whether the response was DNSSEC-protected.
+
+Examples:
+
+mox dns lookup ptr 1.1.1.1
+mox dns lookup mx xmox.nl
+mox dns lookup txt _dmarc.xmox.nl.
+mox dns lookup tlsa _25._tcp.xmox.nl
+
+	usage: mox dns lookup [ptr | mx | cname | ips | a | aaaa | ns | txt | srv | tlsa] name
+
 # mox dkim gened25519
 
 Generate a new ed25519 key for use with DKIM.
@@ -591,7 +707,7 @@ strength. This is convenient because of maximum DNS message sizes. At the time
 of writing, not many mail servers appear to support ed25519 DKIM keys though,
 so it is recommended to sign messages with both RSA and ed25519 keys.
 
-	usage: mox dkim gened25519 >$selector._domainkey.$domain.ed25519key.pkcs8.pem
+	usage: mox dkim gened25519 >$selector._domainkey.$domain.ed25519.privatekey.pkcs8.pem
 
 # mox dkim genrsa
 
@@ -600,7 +716,7 @@ Generate a new 2048 bit RSA private key for use with DKIM.
 The generated file is in PEM format, and has a comment it is generated for use
 with DKIM, by mox.
 
-	usage: mox dkim genrsa >$selector._domainkey.$domain.rsakey.pkcs8.pem
+	usage: mox dkim genrsa >$selector._domainkey.$domain.rsa2048.privatekey.pkcs8.pem
 
 # mox dkim lookup
 
@@ -871,12 +987,13 @@ Ensure messages in the database have a pre-parsed MIME form in the database.
 
 # mox recalculatemailboxcounts
 
-Recalculate message counts for all mailboxes in the account.
+Recalculate message counts for all mailboxes in the account, and total message size for quota.
 
 When a message is added to/removed from a mailbox, or when message flags change,
-the total, unread, unseen and deleted messages are accounted, and the total size
-of the mailbox. In case of a bug in this accounting, the numbers could become
-incorrect. This command will find, fix and print them.
+the total, unread, unseen and deleted messages are accounted, the total size of
+the mailbox, and the total message size for the account. In case of a bug in
+this accounting, the numbers could become incorrect. This command will find, fix
+and print them.
 
 	usage: mox recalculatemailboxcounts account
 
