@@ -7,10 +7,12 @@ import (
 	"io"
 	"net"
 	"os"
+	"path/filepath"
 	"testing"
 	"time"
 
 	"github.com/mjl-/mox/imapclient"
+	"github.com/mjl-/mox/mlog"
 	"github.com/mjl-/mox/mox-"
 	"github.com/mjl-/mox/store"
 )
@@ -57,17 +59,18 @@ func FuzzServer(f *testing.F) {
 		f.Add(tag + cmd)
 	}
 
+	log := mlog.New("imapserver", nil)
 	mox.Context = ctxbg
-	mox.ConfigStaticPath = "../testdata/imapserverfuzz/mox.conf"
+	mox.ConfigStaticPath = filepath.FromSlash("../testdata/imapserverfuzz/mox.conf")
 	mox.MustLoadConfig(true, false)
 	dataDir := mox.ConfigDirPath(mox.Conf.Static.DataDir)
 	os.RemoveAll(dataDir)
-	acc, err := store.OpenAccount("mjl")
+	acc, err := store.OpenAccount(log, "mjl")
 	if err != nil {
 		f.Fatalf("open account: %v", err)
 	}
 	defer acc.Close()
-	err = acc.SetPassword("testtest")
+	err = acc.SetPassword(log, password0)
 	if err != nil {
 		f.Fatalf("set password: %v", err)
 	}
