@@ -11,11 +11,13 @@ import (
 	"encoding/pem"
 	"fmt"
 	golog "log"
+	"log/slog"
 	"math/big"
 	"net"
 	"os"
 	"os/signal"
 	"path/filepath"
+	"runtime"
 	"syscall"
 	"time"
 
@@ -64,30 +66,48 @@ during those commands instead of during "data".
 	if userConfDir == "" {
 		userConfDir = "."
 	}
+	// If we are being run to gather help output, show a placeholder directory
+	// instead of evaluating to the actual userconfigdir on the host os.
+	if c._gather {
+		userConfDir = "$userconfigdir"
+	}
 
 	var dir, ip string
+	var initOnly bool
 	c.flag.StringVar(&dir, "dir", filepath.Join(userConfDir, "mox-localserve"), "configuration storage directory")
 	c.flag.StringVar(&ip, "ip", "", "serve on this ip instead of default 127.0.0.1 and ::1. only used when writing configuration, at first launch.")
+	c.flag.BoolVar(&initOnly, "initonly", false, "write configuration files and exit")
 	args := c.Parse()
 	if len(args) != 0 {
 		c.Usage()
 	}
 
-	log := mlog.New("localserve")
-
+	log := c.log
 	mox.FilesImmediate = true
 
+	if initOnly {
+		if _, err := os.Stat(dir); err == nil {
+			log.Print("warning: directory for configuration files already exists, continuing")
+		}
+		log.Print("creating mox localserve config", slog.String("dir", dir))
+		err := writeLocalConfig(log, dir, ip)
+		if err != nil {
+			log.Fatalx("creating mox localserve config", err, slog.String("dir", dir))
+		}
+		return
+	}
+
 	// Load config, creating a new one if needed.
 	var existingConfig bool
 	if _, err := os.Stat(dir); err != nil && os.IsNotExist(err) {
 		err := writeLocalConfig(log, dir, ip)
 		if err != nil {
-			log.Fatalx("creating mox localserve config", err, mlog.Field("dir", dir))
+			log.Fatalx("creating mox localserve config", err, slog.String("dir", dir))
 		}
 	} else if err != nil {
-		log.Fatalx("stat config dir", err, mlog.Field("dir", dir))
+		log.Fatalx("stat config dir", err, slog.String("dir", dir))
 	} else if err := localLoadConfig(log, dir); err != nil {
-		log.Fatalx("loading mox localserve config (hint: when creating a new config with -dir, the directory must not yet exist)", err, mlog.Field("dir", dir))
+		log.Fatalx("loading mox localserve config (hint: when creating a new config with -dir, the directory must not yet exist)", err, slog.String("dir", dir))
 	} else if ip != "" {
 		log.Fatal("can only use -ip when writing a new config file")
 	} else {
@@ -98,7 +118,7 @@ during those commands instead of during "data".
 		mox.Conf.Log[""] = level
 		mlog.SetConfig(mox.Conf.Log)
 	} else if loglevel != "" && !ok {
-		log.Fatal("unknown loglevel", mlog.Field("loglevel", loglevel))
+		log.Fatal("unknown loglevel", slog.String("loglevel", loglevel))
 	}
 
 	// Initialize receivedid.
@@ -124,11 +144,13 @@ during those commands instead of during "data".
 	queue.Localserve = true
 
 	const mtastsdbRefresher = false
+	const sendDMARCReports = false
+	const sendTLSReports = false
 	const skipForkExec = true
-	if err := start(mtastsdbRefresher, skipForkExec); err != nil {
+	if err := start(mtastsdbRefresher, sendDMARCReports, sendTLSReports, skipForkExec); err != nil {
 		log.Fatalx("starting mox", err)
 	}
-	golog.Printf("mox, version %s", moxvar.Version)
+	golog.Printf("mox, version %s, %s %s/%s", moxvar.Version, runtime.Version(), runtime.GOOS, runtime.GOARCH)
 	golog.Print("")
 	golog.Printf("the default user is mox@localhost, with password moxmoxmox")
 	golog.Printf("the default admin password is moxadmin")
@@ -143,17 +165,17 @@ during those commands instead of during "data".
 	golog.Print("")
 	golog.Printf(`if the localpart begins with "mailfrom" or "rcptto", the error is returned during those commands instead of during "data"`)
 	golog.Print("")
-	golog.Print(" smtp://localhost:1025                                    - receive email")
-	golog.Print("smtps://mox%40localhost:moxmoxmox@localhost:1465          - send email")
-	golog.Print(" smtp://mox%40localhost:moxmoxmox@localhost:1587          - send email (without tls)")
-	golog.Print("imaps://mox%40localhost:moxmoxmox@localhost:1993          - read email")
-	golog.Print(" imap://mox%40localhost:moxmoxmox@localhost:1143          - read email (without tls)")
-	golog.Print("https://mox%40localhost:moxmoxmox@localhost:1443/account/ - account https")
-	golog.Print(" http://mox%40localhost:moxmoxmox@localhost:1080/account/ - account http (without tls)")
-	golog.Print("https://mox%40localhost:moxmoxmox@localhost:1443/webmail/ - webmail https")
-	golog.Print(" http://mox%40localhost:moxmoxmox@localhost:1080/webmail/ - webmail http (without tls)")
-	golog.Print("https://admin:moxadmin@localhost:1443/admin/              - admin https")
-	golog.Print(" http://admin:moxadmin@localhost:1080/admin/              - admin http (without tls)")
+	golog.Print(" smtp://localhost:1025                           - receive email")
+	golog.Print("smtps://mox%40localhost:moxmoxmox@localhost:1465 - send email")
+	golog.Print(" smtp://mox%40localhost:moxmoxmox@localhost:1587 - send email (without tls)")
+	golog.Print("imaps://mox%40localhost:moxmoxmox@localhost:1993 - read email")
+	golog.Print(" imap://mox%40localhost:moxmoxmox@localhost:1143 - read email (without tls)")
+	golog.Print("https://localhost:1443/account/                  - account https (email mox@localhost, password moxmoxmox)")
+	golog.Print(" http://localhost:1080/account/                  - account http (without tls)")
+	golog.Print("https://localhost:1443/webmail/                  - webmail https (email mox@localhost, password moxmoxmox)")
+	golog.Print(" http://localhost:1080/webmail/                  - webmail http (without tls)")
+	golog.Print("https://localhost:1443/admin/                    - admin https (password moxadmin)")
+	golog.Print(" http://localhost:1080/admin/                    - admin http (without tls)")
 	golog.Print("")
 	if existingConfig {
 		golog.Printf("serving from existing config dir %s/", dir)
@@ -185,7 +207,7 @@ during those commands instead of during "data".
 	sigc := make(chan os.Signal, 1)
 	signal.Notify(sigc, os.Interrupt, syscall.SIGTERM)
 	sig := <-sigc
-	log.Print("shutting down, waiting max 3s for existing connections", mlog.Field("signal", sig))
+	log.Print("shutting down, waiting max 3s for existing connections", slog.Any("signal", sig))
 	shutdown(log)
 	if num, ok := sig.(syscall.Signal); ok {
 		os.Exit(int(num))
@@ -194,7 +216,7 @@ during those commands instead of during "data".
 	}
 }
 
-func writeLocalConfig(log *mlog.Log, dir, ip string) (rerr error) {
+func writeLocalConfig(log mlog.Log, dir, ip string) (rerr error) {
 	defer func() {
 		x := recover()
 		if x != nil {
@@ -204,7 +226,7 @@ func writeLocalConfig(log *mlog.Log, dir, ip string) (rerr error) {
 		}
 		if rerr != nil {
 			err := os.RemoveAll(dir)
-			log.Check(err, "removing config directory", mlog.Field("dir", dir))
+			log.Check(err, "removing config directory", slog.String("dir", dir))
 		}
 	}()
 
@@ -321,11 +343,15 @@ func writeLocalConfig(log *mlog.Log, dir, ip string) (rerr error) {
 	local.WebserverHTTPS.Enabled = true
 	local.WebserverHTTPS.Port = 1443
 
+	uid := os.Getuid()
+	if uid < 0 {
+		uid = 1 // For windows.
+	}
 	static := config.Static{
 		DataDir:           ".",
 		LogLevel:          "traceauth",
 		Hostname:          "localhost",
-		User:              fmt.Sprintf("%d", os.Getuid()),
+		User:              fmt.Sprintf("%d", uid),
 		AdminPasswordFile: "adminpasswd",
 		Pedantic:          true,
 		Listeners: map[string]config.Listener{
@@ -410,10 +436,10 @@ func writeLocalConfig(log *mlog.Log, dir, ip string) (rerr error) {
 	xcheck(err, "loading config")
 
 	// Set password on account.
-	a, _, err := store.OpenEmail("mox@localhost")
+	a, _, err := store.OpenEmail(log, "mox@localhost")
 	xcheck(err, "opening account to set password")
 	password := "moxmoxmox"
-	err = a.SetPassword(password)
+	err = a.SetPassword(log, password)
 	xcheck(err, "setting password")
 	err = a.Close()
 	xcheck(err, "closing account")
@@ -422,10 +448,10 @@ func writeLocalConfig(log *mlog.Log, dir, ip string) (rerr error) {
 	return nil
 }
 
-func localLoadConfig(log *mlog.Log, dir string) error {
+func localLoadConfig(log mlog.Log, dir string) error {
 	mox.ConfigStaticPath = filepath.Join(dir, "mox.conf")
 	mox.ConfigDynamicPath = filepath.Join(dir, "domains.conf")
-	errs := mox.LoadConfig(context.Background(), true, false)
+	errs := mox.LoadConfig(context.Background(), log, true, false)
 	if len(errs) > 1 {
 		log.Error("loading config generated config file: multiple errors")
 		for _, err := range errs {
