@@ -3,6 +3,7 @@ package message
 import (
 	"fmt"
 	"io"
+	"log/slog"
 	"net/textproto"
 
 	"github.com/mjl-/mox/dns"
@@ -17,28 +18,34 @@ import (
 // From headers may be present. From returns an error if there is not exactly
 // one address. This address can be used for evaluating a DMARC policy against
 // SPF and DKIM results.
-func From(log *mlog.Log, strict bool, r io.ReaderAt) (raddr smtp.Address, header textproto.MIMEHeader, rerr error) {
+func From(elog *slog.Logger, strict bool, r io.ReaderAt) (raddr smtp.Address, envelope *Envelope, header textproto.MIMEHeader, rerr error) {
+	log := mlog.New("message", elog)
+
 	// ../rfc/7489:1243
 
 	// todo: only allow utf8 if enabled in session/message?
 
-	p, err := Parse(log, strict, r)
+	p, err := Parse(log.Logger, strict, r)
 	if err != nil {
 		// todo: should we continue with p, perhaps headers can be parsed?
-		return raddr, nil, fmt.Errorf("parsing message: %v", err)
+		return raddr, nil, nil, fmt.Errorf("parsing message: %v", err)
 	}
 	header, err = p.Header()
 	if err != nil {
-		return raddr, nil, fmt.Errorf("parsing message header: %v", err)
+		return raddr, nil, nil, fmt.Errorf("parsing message header: %v", err)
 	}
 	from := p.Envelope.From
 	if len(from) != 1 {
-		return raddr, nil, fmt.Errorf("from header has %d addresses, need exactly 1 address", len(from))
+		return raddr, nil, nil, fmt.Errorf("from header has %d addresses, need exactly 1 address", len(from))
 	}
 	d, err := dns.ParseDomain(from[0].Host)
 	if err != nil {
-		return raddr, nil, fmt.Errorf("bad domain in from address: %v", err)
+		return raddr, nil, nil, fmt.Errorf("bad domain in from address: %v", err)
 	}
-	addr := smtp.Address{Localpart: smtp.Localpart(from[0].User), Domain: d}
-	return addr, textproto.MIMEHeader(header), nil
+	lp, err := smtp.ParseLocalpart(from[0].User)
+	if err != nil {
+		return raddr, nil, nil, fmt.Errorf("parsing localpart in from address: %v", err)
+	}
+	addr := smtp.Address{Localpart: lp, Domain: d}
+	return addr, p.Envelope, textproto.MIMEHeader(header), nil
 }
