@@ -36,15 +36,27 @@ func FindAccount(localpart smtp.Localpart, domain dns.Domain, allowPostmaster bo
 		return false
 	}
 
+	// Check for special mail host addresses.
 	if localpart == "postmaster" && postmasterDomain() {
 		if !allowPostmaster {
 			return "", "", config.Destination{}, ErrAccountNotFound
 		}
 		return Conf.Static.Postmaster.Account, "postmaster", config.Destination{Mailbox: Conf.Static.Postmaster.Mailbox}, nil
 	}
+	if localpart == Conf.Static.HostTLSRPT.ParsedLocalpart && domain == Conf.Static.HostnameDomain {
+		// Get destination, should always be present.
+		canonical := smtp.NewAddress(localpart, domain).String()
+		accAddr, ok := Conf.AccountDestination(canonical)
+		if !ok {
+			return "", "", config.Destination{}, ErrAccountNotFound
+		}
+		return accAddr.Account, canonical, accAddr.Destination, nil
+	}
 
 	d, ok := Conf.Domain(domain)
-	if !ok {
+	if !ok || d.ReportsOnly {
+		// For ReportsOnly, we also return ErrDomainNotFound, so this domain isn't
+		// considered local/authoritative during delivery.
 		return "", "", config.Destination{}, ErrDomainNotFound
 	}
 
@@ -73,9 +85,6 @@ func CanonicalLocalpart(localpart smtp.Localpart, d config.Domain) (smtp.Localpa
 	if d.LocalpartCatchallSeparator != "" {
 		t := strings.SplitN(string(localpart), d.LocalpartCatchallSeparator, 2)
 		localpart = smtp.Localpart(t[0])
-		if localpart == "" {
-			return "", fmt.Errorf("empty localpart")
-		}
 	}
 
 	if !d.LocalpartCaseSensitive {
