@@ -3,6 +3,13 @@ package main
 import (
 	"bytes"
 	"context"
+	"crypto"
+	"crypto/ecdsa"
+	"crypto/elliptic"
+	cryptorand "crypto/rand"
+	"crypto/rsa"
+	"crypto/x509"
+	"encoding/pem"
 	"errors"
 	"fmt"
 	"log"
@@ -24,6 +31,7 @@ import (
 	"github.com/mjl-/mox/config"
 	"github.com/mjl-/mox/dns"
 	"github.com/mjl-/mox/dnsbl"
+	"github.com/mjl-/mox/mlog"
 	"github.com/mjl-/mox/mox-"
 	"github.com/mjl-/mox/smtp"
 	"github.com/mjl-/mox/store"
@@ -33,11 +41,19 @@ import (
 var moxService string
 
 func pwgen() string {
-	rand := mox.NewRand()
 	chars := "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!@#$%^&*-_;:,<.>/"
 	s := ""
+	buf := make([]byte, 1)
 	for i := 0; i < 12; i++ {
-		s += string(chars[rand.Intn(len(chars))])
+		for {
+			cryptorand.Read(buf)
+			i := int(buf[0])
+			if i+len(chars) > 255 {
+				continue // Prevent bias.
+			}
+			s += string(chars[i%len(chars)])
+			break
+		}
 	}
 	return s
 }
@@ -146,6 +162,39 @@ logging in with IMAP.
 	resolveCtx, resolveCancel := context.WithTimeout(context.Background(), 10*time.Second)
 	defer resolveCancel()
 
+	// Some DNSSEC-verifying resolvers return unauthentic data for ".", so we check "com".
+	fmt.Printf("Checking if DNS resolvers are DNSSEC-verifying...")
+	_, resolverDNSSECResult, err := resolver.LookupNS(resolveCtx, "com.")
+	if err != nil {
+		fmt.Println("")
+		fatalf("checking dnssec support in resolver: %v", err)
+	} else if !resolverDNSSECResult.Authentic {
+		fmt.Printf(`
+
+WARNING: It looks like the DNS resolvers configured on your system do not
+verify DNSSEC, or aren't trusted (by having loopback IPs or through "options
+trust-ad" in /etc/resolv.conf).  Without DNSSEC, outbound delivery with SMTP
+used unprotected MX records, and SMTP STARTTLS connections cannot verify the TLS
+certificate with DANE (based on a public key in DNS), and will fall back to
+either MTA-STS for verification, or use "opportunistic TLS" with no certificate
+verification.
+
+Recommended action: Install unbound, a DNSSEC-verifying recursive DNS resolver,
+ensure it has DNSSEC root keys (see unbound-anchor), and enable support for
+"extended dns errors" (EDE, available since unbound v1.16.0). Test with
+"dig com. ns" and look for "ad" (authentic data) in response "flags".
+
+cat <<EOF >/etc/unbound/unbound.conf.d/ede.conf
+server:
+    ede: yes
+    val-log-level: 2
+EOF
+
+`)
+	} else {
+		fmt.Println(" OK")
+	}
+
 	// We are going to find the (public) IPs to listen on and possibly the host name.
 
 	// Start with reasonable defaults. We'll replace them specific IPs, if we can find them.
@@ -244,7 +293,7 @@ logging in with IMAP.
 			for _, ip := range publicIPs {
 				revctx, revcancel := context.WithTimeout(resolveCtx, 5*time.Second)
 				defer revcancel()
-				l, err := resolver.LookupAddr(revctx, ip)
+				l, _, err := resolver.LookupAddr(revctx, ip)
 				if err != nil {
 					warnf("WARNING: looking up reverse name(s) for %s: %v", ip, err)
 				}
@@ -306,7 +355,7 @@ again with the -hostname flag.
 	fmt.Printf("Looking up IPs for hostname %s...", dnshostname)
 	ipctx, ipcancel := context.WithTimeout(resolveCtx, 5*time.Second)
 	defer ipcancel()
-	ips, err := resolver.LookupIPAddr(ipctx, dnshostname.ASCII+".")
+	ips, domainDNSSECResult, err := resolver.LookupIPAddr(ipctx, dnshostname.ASCII+".")
 	ipcancel()
 	var xips []net.IPAddr
 	var hostIPs []string
@@ -403,6 +452,23 @@ This likely means one of two things:
 
 
 `, dnshostname, err)
+	} else if !domainDNSSECResult.Authentic {
+		if !dnswarned {
+			fmt.Printf("\n")
+		}
+		dnswarned = true
+		fmt.Printf(`
+NOTE: It looks like the DNS records of your domain (zone) are not DNSSEC-signed.
+Mail servers that send email to your domain, or receive email from your domain,
+cannot verify that the MX/SPF/DKIM/DMARC/MTA-STS records they receive are
+authentic. DANE, for authenticated delivery without relying on a pool of
+certificate authorities, requires DNSSEC, so will not be configured at this
+time.
+Recommended action: Continue now, but consider enabling DNSSEC for your domain
+later at your DNS operator, and adding DANE records for protecting incoming
+messages over SMTP.
+
+`)
 	}
 
 	if !dnswarned {
@@ -421,7 +487,7 @@ This likely means one of two things:
 			go func() {
 				revctx, revcancel := context.WithTimeout(resolveCtx, 5*time.Second)
 				defer revcancel()
-				addrs, err := resolver.LookupAddr(revctx, s)
+				addrs, _, err := resolver.LookupAddr(revctx, s)
 				results <- result{s, addrs, err}
 			}()
 		}
@@ -473,7 +539,7 @@ This likely means one of two things:
 		for _, zone := range zones {
 			for _, ip := range hostIPs {
 				dnsblctx, dnsblcancel := context.WithTimeout(resolveCtx, 5*time.Second)
-				status, expl, err := dnsbl.Lookup(dnsblctx, resolver, zone, net.ParseIP(ip))
+				status, expl, err := dnsbl.Lookup(dnsblctx, c.log.Logger, resolver, zone, net.ParseIP(ip))
 				dnsblcancel()
 				if status == dnsbl.StatusPass {
 					continue
@@ -538,7 +604,7 @@ many authentication failures).
 
 	dc := config.Dynamic{}
 	sc := config.Static{
-		DataDir:           "../data",
+		DataDir:           filepath.FromSlash("../data"),
 		User:              user,
 		LogLevel:          "debug", // Help new users, they'll bring it back to info when it all works.
 		Hostname:          dnshostname.Name(),
@@ -547,8 +613,9 @@ many authentication failures).
 	if !existingWebserver {
 		sc.ACME = map[string]config.ACME{
 			"letsencrypt": {
-				DirectoryURL: "https://acme-v02.api.letsencrypt.org/directory",
-				ContactEmail: args[0], // todo: let user specify an alternative fallback address?
+				DirectoryURL:     "https://acme-v02.api.letsencrypt.org/directory",
+				ContactEmail:     args[0], // todo: let user specify an alternative fallback address?
+				IssuerDomainName: "letsencrypt.org",
 			},
 		}
 	}
@@ -571,9 +638,9 @@ many authentication failures).
 	public.IMAPS.Enabled = true
 
 	if existingWebserver {
-		hostbase := fmt.Sprintf("path/to/%s", dnshostname.Name())
-		mtastsbase := fmt.Sprintf("path/to/mta-sts.%s", domain.Name())
-		autoconfigbase := fmt.Sprintf("path/to/autoconfig.%s", domain.Name())
+		hostbase := filepath.FromSlash("path/to/" + dnshostname.Name())
+		mtastsbase := filepath.FromSlash("path/to/mta-sts." + domain.Name())
+		autoconfigbase := filepath.FromSlash("path/to/autoconfig." + domain.Name())
 		public.TLS = &config.TLS{
 			KeyCerts: []config.KeyCert{
 				{CertFile: hostbase + "-chain.crt.pem", KeyFile: hostbase + ".key.pem"},
@@ -581,9 +648,57 @@ many authentication failures).
 				{CertFile: autoconfigbase + "-chain.crt.pem", KeyFile: autoconfigbase + ".key.pem"},
 			},
 		}
+
+		fmt.Println(
+			`Placeholder paths to TLS certificates to be provided by the existing webserver
+have been placed in config/mox.conf and need to be edited.
+
+No private keys for the public listener have been generated for use with DANE.
+To configure DANE (which requires DNSSEC), set config field HostPrivateKeyFiles
+in the "public" Listener to both RSA 2048-bit and ECDSA P-256 private key files
+and check the admin page for the needed DNS records.`)
+
 	} else {
+		// todo: we may want to generate a second set of keys, make the user already add it to the DNS, but keep the private key offline. would require config option to specify a public key only, so the dane records can be generated.
+		hostRSAPrivateKey, err := rsa.GenerateKey(cryptorand.Reader, 2048)
+		if err != nil {
+			fatalf("generating rsa private key for host: %s", err)
+		}
+		hostECDSAPrivateKey, err := ecdsa.GenerateKey(elliptic.P256(), cryptorand.Reader)
+		if err != nil {
+			fatalf("generating ecsa private key for host: %s", err)
+		}
+		now := time.Now()
+		timestamp := now.Format("20060102T150405")
+		hostRSAPrivateKeyFile := filepath.Join("hostkeys", fmt.Sprintf("%s.%s.%s.privatekey.pkcs8.pem", dnshostname.Name(), timestamp, "rsa2048"))
+		hostECDSAPrivateKeyFile := filepath.Join("hostkeys", fmt.Sprintf("%s.%s.%s.privatekey.pkcs8.pem", dnshostname.Name(), timestamp, "ecdsap256"))
+		xwritehostkeyfile := func(path string, key crypto.Signer) {
+			buf, err := x509.MarshalPKCS8PrivateKey(key)
+			if err != nil {
+				fatalf("marshaling host private key to pkcs8 for %s: %s", path, err)
+			}
+			var b bytes.Buffer
+			block := pem.Block{
+				Type:  "PRIVATE KEY",
+				Bytes: buf,
+			}
+			err = pem.Encode(&b, &block)
+			if err != nil {
+				fatalf("pem-encoding host private key file for %s: %s", path, err)
+			}
+			xwritefile(path, b.Bytes(), 0600)
+		}
+		xwritehostkeyfile(filepath.Join("config", hostRSAPrivateKeyFile), hostRSAPrivateKey)
+		xwritehostkeyfile(filepath.Join("config", hostECDSAPrivateKeyFile), hostECDSAPrivateKey)
+
 		public.TLS = &config.TLS{
 			ACME: "letsencrypt",
+			HostPrivateKeyFiles: []string{
+				hostRSAPrivateKeyFile,
+				hostECDSAPrivateKeyFile,
+			},
+			HostPrivateRSA2048Keys:   []crypto.Signer{hostRSAPrivateKey},
+			HostPrivateECDSAP256Keys: []crypto.Signer{hostECDSAPrivateKey},
 		}
 		public.AutoconfigHTTPS.Enabled = true
 		public.MTASTSHTTPS.Enabled = true
@@ -596,18 +711,26 @@ many authentication failures).
 		public.SMTP.DNSBLs = append(public.SMTP.DNSBLs, zone.Name())
 	}
 
+	// Monitor DNSBLs by default, without using them for incoming deliveries.
+	for _, zone := range zones {
+		dc.MonitorDNSBLs = append(dc.MonitorDNSBLs, zone.Name())
+	}
+
 	internal := config.Listener{
 		IPs:      privateListenerIPs,
 		Hostname: "localhost",
 	}
 	internal.AccountHTTP.Enabled = true
 	internal.AdminHTTP.Enabled = true
-	internal.MetricsHTTP.Enabled = true
 	internal.WebmailHTTP.Enabled = true
+	internal.MetricsHTTP.Enabled = true
 	if existingWebserver {
 		internal.AccountHTTP.Port = 1080
+		internal.AccountHTTP.Forwarded = true
 		internal.AdminHTTP.Port = 1080
+		internal.AdminHTTP.Forwarded = true
 		internal.WebmailHTTP.Port = 1080
+		internal.WebmailHTTP.Forwarded = true
 		internal.AutoconfigHTTPS.Enabled = true
 		internal.AutoconfigHTTPS.Port = 81
 		internal.AutoconfigHTTPS.NonTLS = true
@@ -624,9 +747,12 @@ many authentication failures).
 	}
 	sc.Postmaster.Account = accountName
 	sc.Postmaster.Mailbox = "Postmaster"
+	sc.HostTLSRPT.Account = accountName
+	sc.HostTLSRPT.Localpart = "tls-reports"
+	sc.HostTLSRPT.Mailbox = "TLSRPT"
 
-	mox.ConfigStaticPath = "config/mox.conf"
-	mox.ConfigDynamicPath = "config/domains.conf"
+	mox.ConfigStaticPath = filepath.FromSlash("config/mox.conf")
+	mox.ConfigDynamicPath = filepath.FromSlash("config/domains.conf")
 
 	mox.Conf.DynamicLastCheck = time.Now() // Prevent error logging by Make calls below.
 
@@ -657,7 +783,7 @@ many authentication failures).
 	for _, bl := range public.SMTP.DNSBLs {
 		confstr = strings.ReplaceAll(confstr, "- "+bl+"\n", "#- "+bl+"\n")
 	}
-	xwritefile("config/mox.conf", []byte(confstr), 0660)
+	xwritefile(filepath.FromSlash("config/mox.conf"), []byte(confstr), 0660)
 
 	// Generate domains config, and add a commented out example for delivery to a mailing list.
 	var db bytes.Buffer
@@ -696,11 +822,11 @@ many authentication failures).
 		ndests += "#\t\t" + line + "\n"
 	}
 	dconfstr := strings.ReplaceAll(db.String(), odests, ndests)
-	xwritefile("config/domains.conf", []byte(dconfstr), 0660)
+	xwritefile(filepath.FromSlash("config/domains.conf"), []byte(dconfstr), 0660)
 
 	// Verify config.
 	loadTLSKeyCerts := !existingWebserver
-	mc, errs := mox.ParseConfig(context.Background(), "config/mox.conf", true, loadTLSKeyCerts, false)
+	mc, errs := mox.ParseConfig(context.Background(), c.log, filepath.FromSlash("config/mox.conf"), true, loadTLSKeyCerts, false)
 	if len(errs) > 0 {
 		if len(errs) > 1 {
 			log.Printf("checking generated config, multiple errors:")
@@ -721,16 +847,24 @@ many authentication failures).
 		fatalf("cannot find domain in new config")
 	}
 
-	acc, _, err := store.OpenEmail(args[0])
+	acc, _, err := store.OpenEmail(c.log, args[0])
 	if err != nil {
 		fatalf("open account: %s", err)
 	}
 	cleanupPaths = append(cleanupPaths, dataDir, filepath.Join(dataDir, "accounts"), filepath.Join(dataDir, "accounts", accountName), filepath.Join(dataDir, "accounts", accountName, "index.db"))
 
 	password := pwgen()
-	if err := acc.SetPassword(password); err != nil {
+
+	// Kludge to cause no logging to be printed about setting a new password.
+	loglevel := mox.Conf.Log[""]
+	mox.Conf.Log[""] = mlog.LevelWarn
+	mlog.SetConfig(mox.Conf.Log)
+	if err := acc.SetPassword(c.log, password); err != nil {
 		fatalf("setting password: %s", err)
 	}
+	mox.Conf.Log[""] = loglevel
+	mlog.SetConfig(mox.Conf.Log)
+
 	if err := acc.Close(); err != nil {
 		fatalf("closing account: %s", err)
 	}
@@ -745,8 +879,9 @@ autoconfig/autodiscover does not work, use these settings:
 Configuration files have been written to config/mox.conf and
 config/domains.conf.
 
-Create the DNS records below. The admin interface can show these same records, and
-has a page to check they have been configured correctly.
+Create the DNS records below, by adding them to your zone file or through the
+web interface of your DNS operator. The admin interface can show these same
+records, and has a page to check they have been configured correctly.
 
 You must configure your existing webserver to forward requests for:
 
@@ -765,14 +900,17 @@ The paths are relative to config/ directory that holds mox.conf! To test if your
 config is valid, run:
 
 	./mox config test
+
+The DNS records to add:
 `, domain.ASCII, domain.ASCII, dnshostname.ASCII)
 	} else {
 		fmt.Printf(`
 Configuration files have been written to config/mox.conf and
-config/domains.conf. You should review them. Then create the DNS records below.
-You can also skip creating the DNS records and start mox immediately. The admin
-interface can show these same records, and has a page to check they have been
-configured correctly.
+config/domains.conf. You should review them. Then create the DNS records below,
+by adding them to your zone file or through the web interface of your DNS
+operator. You can also skip creating the DNS records and start mox immediately.
+The admin interface can show these same records, and has a page to check they
+have been configured correctly. The DNS records to add:
 `)
 	}
 
@@ -780,7 +918,7 @@ configured correctly.
 	// priming dns caches with negative/absent records, causing our "quick setup" to
 	// appear to fail or take longer than "quick".
 
-	records, err := mox.DomainRecords(confDomain, domain)
+	records, err := mox.DomainRecords(confDomain, domain, domainDNSSECResult.Authentic, "letsencrypt.org", "")
 	if err != nil {
 		fatalf("making required DNS records")
 	}
@@ -837,9 +975,6 @@ To access these from your browser, run
 "ssh -L 8080:localhost:80 you@yourmachine" locally and open
 http://localhost:8080/[...].
 
-For secure email exchange you should have a strictly validating DNSSEC
-resolver. An easy and the recommended way is to install unbound.
-
 If you run into problem, have questions/feedback or found a bug, please let us
 know. Mox needs your help!
 
