@@ -3,8 +3,10 @@ package smtpserver
 import (
 	"context"
 	"fmt"
+	"log/slog"
 	"net"
 	"os"
+	"strings"
 	"time"
 
 	"github.com/mjl-/bstore"
@@ -15,8 +17,10 @@ import (
 	"github.com/mjl-/mox/dns"
 	"github.com/mjl-/mox/dnsbl"
 	"github.com/mjl-/mox/iprev"
+	"github.com/mjl-/mox/message"
 	"github.com/mjl-/mox/mlog"
 	"github.com/mjl-/mox/mox-"
+	"github.com/mjl-/mox/publicsuffix"
 	"github.com/mjl-/mox/smtp"
 	"github.com/mjl-/mox/store"
 	"github.com/mjl-/mox/subjectpass"
@@ -24,10 +28,13 @@ import (
 )
 
 type delivery struct {
+	tls         bool
 	m           *store.Message
 	dataFile    *os.File
 	rcptAcc     rcptAccount
 	acc         *store.Account
+	msgTo       []message.Address
+	msgCc       []message.Address
 	msgFrom     smtp.Address
 	dnsBLs      []dns.Domain
 	dmarcUse    bool
@@ -37,16 +44,20 @@ type delivery struct {
 }
 
 type analysis struct {
-	accept      bool
-	mailbox     string
-	code        int
-	secode      string
-	userError   bool
-	errmsg      string
-	err         error              // For our own logging, not sent to remote.
-	dmarcReport *dmarcrpt.Feedback // Validated dmarc aggregate report, not yet stored.
-	tlsReport   *tlsrpt.Report     // Validated TLS report, not yet stored.
-	reason      string             // If non-empty, reason for this decision. Can be one of reputationMethod and a few other tokens.
+	accept              bool
+	mailbox             string
+	code                int
+	secode              string
+	userError           bool
+	errmsg              string
+	err                 error              // For our own logging, not sent to remote.
+	dmarcReport         *dmarcrpt.Feedback // Validated DMARC aggregate report, not yet stored.
+	tlsReport           *tlsrpt.Report     // Validated TLS report, not yet stored.
+	reason              string             // If non-empty, reason for this decision. Can be one of reputationMethod and a few other tokens.
+	dmarcOverrideReason string             // If set, one of dmarcrpt.PolicyOverride
+	// Additional headers to add during delivery. Used for reasons a message to a
+	// dmarc/tls reporting address isn't processed.
+	headers string
 }
 
 const (
@@ -64,10 +75,24 @@ const (
 	reasonDNSBlocklisted    = "dns-blocklisted"
 	reasonSubjectpass       = "subjectpass"
 	reasonSubjectpassError  = "subjectpass-error"
-	reasonIPrev             = "iprev" // No or mil junk reputation signals, and bad iprev.
+	reasonIPrev             = "iprev" // No or mild junk reputation signals, and bad iprev.
 )
 
-func analyze(ctx context.Context, log *mlog.Log, resolver dns.Resolver, d delivery) analysis {
+func isListDomain(d delivery, ld dns.Domain) bool {
+	if d.m.MailFromValidated && ld.Name() == d.m.MailFromDomain {
+		return true
+	}
+	for _, r := range d.dkimResults {
+		if r.Status == dkim.StatusPass && r.Sig.Domain == ld {
+			return true
+		}
+	}
+	return false
+}
+
+func analyze(ctx context.Context, log mlog.Log, resolver dns.Resolver, d delivery) analysis {
+	var headers string
+
 	mailbox := d.rcptAcc.destination.Mailbox
 	if mailbox == "" {
 		mailbox = "Inbox"
@@ -80,18 +105,15 @@ func analyze(ctx context.Context, log *mlog.Log, resolver dns.Resolver, d delive
 		mailbox = rs.Mailbox
 	}
 	if rs != nil && !rs.ListAllowDNSDomain.IsZero() {
-		ld := rs.ListAllowDNSDomain
 		// todo: on temporary failures, reject temporarily?
-		if d.m.MailFromValidated && ld.Name() == d.m.MailFromDomain {
-			return analysis{accept: true, mailbox: mailbox, reason: reasonListAllow}
-		}
-		for _, r := range d.dkimResults {
-			if r.Status == dkim.StatusPass && r.Sig.Domain == ld {
-				return analysis{accept: true, mailbox: mailbox, reason: reasonListAllow}
-			}
+		if isListDomain(d, rs.ListAllowDNSDomain) {
+			d.m.IsMailingList = true
+			return analysis{accept: true, mailbox: mailbox, reason: reasonListAllow, dmarcOverrideReason: string(dmarcrpt.PolicyOverrideMailingList), headers: headers}
 		}
 	}
 
+	var dmarcOverrideReason string
+
 	// For forwarded messages, we have different junk analysis. We don't reject for
 	// failing DMARC, and we clear fields that could implicate the forwarding mail
 	// server during future classifications on incoming messages (the forwarding mail
@@ -113,6 +135,7 @@ func analyze(ctx context.Context, log *mlog.Log, resolver dns.Resolver, d delive
 			}
 		}
 		d.m.DKIMDomains = dkimdoms
+		dmarcOverrideReason = string(dmarcrpt.PolicyOverrideForwarded)
 		log.Info("forwarded message, clearing identifying signals of forwarding mail server")
 	}
 
@@ -136,7 +159,7 @@ func analyze(ctx context.Context, log *mlog.Log, resolver dns.Resolver, d delive
 			d.m.MailboxID = mb.ID
 			d.m.MailboxDestinedID = mb.ID
 		} else {
-			log.Debug("mailbox not found in database", mlog.Field("mailbox", mailbox))
+			log.Debug("mailbox not found in database", slog.String("mailbox", mailbox))
 		}
 		return nil
 	}
@@ -154,7 +177,7 @@ func analyze(ctx context.Context, log *mlog.Log, resolver dns.Resolver, d delive
 				})
 			})
 			if mberr != nil {
-				return analysis{false, mailbox, smtp.C451LocalErr, smtp.SeSys3Other0, false, "error processing", err, nil, nil, reasonReputationError}
+				return analysis{false, mailbox, smtp.C451LocalErr, smtp.SeSys3Other0, false, "error processing", err, nil, nil, reasonReputationError, dmarcOverrideReason, headers}
 			}
 			d.m.MailboxID = 0 // We plan to reject, no need to set intended MailboxID.
 		}
@@ -168,7 +191,7 @@ func analyze(ctx context.Context, log *mlog.Log, resolver dns.Resolver, d delive
 			d.m.Seen = true
 			log.Info("accepting reject to configured mailbox due to ruleset")
 		}
-		return analysis{accept, mailbox, code, secode, err == nil, errmsg, err, nil, nil, reason}
+		return analysis{accept, mailbox, code, secode, err == nil, errmsg, err, nil, nil, reason, dmarcOverrideReason, headers}
 	}
 
 	if d.dmarcUse && d.dmarcResult.Reject {
@@ -180,17 +203,22 @@ func analyze(ctx context.Context, log *mlog.Log, resolver dns.Resolver, d delive
 	// track of the report. We'll check reputation, defaulting to accept.
 	var dmarcReport *dmarcrpt.Feedback
 	if d.rcptAcc.destination.DMARCReports {
-		// Messages with DMARC aggregate reports must have a dmarc pass. ../rfc/7489:1866
+		// Messages with DMARC aggregate reports must have a DMARC pass. ../rfc/7489:1866
 		if d.dmarcResult.Status != dmarc.StatusPass {
-			log.Info("received dmarc report without dmarc pass, not processing as dmarc report")
-		} else if report, err := dmarcrpt.ParseMessageReport(log, store.FileMsgReader(d.m.MsgPrefix, d.dataFile)); err != nil {
-			log.Infox("parsing dmarc report", err)
+			log.Info("received dmarc aggregate report without dmarc pass, not processing as dmarc report")
+			headers += "X-Mox-DMARCReport-Error: no DMARC pass\r\n"
+		} else if report, err := dmarcrpt.ParseMessageReport(log.Logger, store.FileMsgReader(d.m.MsgPrefix, d.dataFile)); err != nil {
+			log.Infox("parsing dmarc aggregate report", err)
+			headers += "X-Mox-DMARCReport-Error: could not parse report\r\n"
 		} else if d, err := dns.ParseDomain(report.PolicyPublished.Domain); err != nil {
-			log.Infox("parsing domain in dmarc report", err)
+			log.Infox("parsing domain in dmarc aggregate report", err)
+			headers += "X-Mox-DMARCReport-Error: could not parse domain in published policy\r\n"
 		} else if _, ok := mox.Conf.Domain(d); !ok {
-			log.Info("dmarc report for domain not configured, ignoring", mlog.Field("domain", d))
+			log.Info("dmarc aggregate report for domain not configured, ignoring", slog.Any("domain", d))
+			headers += "X-Mox-DMARCReport-Error: published policy domain unrecognized\r\n"
 		} else if report.ReportMetadata.DateRange.End > time.Now().Unix()+60 {
-			log.Info("dmarc report with end date in the future, ignoring", mlog.Field("domain", d), mlog.Field("end", time.Unix(report.ReportMetadata.DateRange.End, 0)))
+			log.Info("dmarc aggregate report with end date in the future, ignoring", slog.Any("domain", d), slog.Time("end", time.Unix(report.ReportMetadata.DateRange.End, 0)))
+			headers += "X-Mox-DMARCReport-Error: report has end date in the future\r\n"
 		} else {
 			dmarcReport = report
 		}
@@ -199,7 +227,12 @@ func analyze(ctx context.Context, log *mlog.Log, resolver dns.Resolver, d delive
 	// Similar to DMARC reporting, we check for the required DKIM. We'll check
 	// reputation, defaulting to accept.
 	var tlsReport *tlsrpt.Report
-	if d.rcptAcc.destination.TLSReports {
+	if d.rcptAcc.destination.HostTLSReports || d.rcptAcc.destination.DomainTLSReports {
+		matchesDomain := func(sigDomain dns.Domain) bool {
+			// RFC seems to require exact DKIM domain match with submitt and message From, we
+			// also allow msgFrom to be subdomain. ../rfc/8460:322
+			return sigDomain == d.msgFrom.Domain || strings.HasSuffix(d.msgFrom.Domain.ASCII, "."+sigDomain.ASCII) && publicsuffix.Lookup(ctx, log.Logger, d.msgFrom.Domain) == publicsuffix.Lookup(ctx, log.Logger, sigDomain)
+		}
 		// Valid DKIM signature for domain must be present. We take "valid" to assume
 		// "passing", not "syntactically valid". We also check for "tlsrpt" as service.
 		// This check is optional, but if anyone goes through the trouble to explicitly
@@ -207,7 +240,14 @@ func analyze(ctx context.Context, log *mlog.Log, resolver dns.Resolver, d delive
 		// ../rfc/8460:320
 		ok := false
 		for _, r := range d.dkimResults {
-			if r.Status == dkim.StatusPass && r.Sig.Domain == d.msgFrom.Domain && r.Sig.Length < 0 && r.Record.ServiceAllowed("tlsrpt") {
+			// The record should have an allowed service "tlsrpt". The RFC mentions it as if
+			// the service must be specified explicitly, but the default allowed services for a
+			// DKIM record are "*", which includes "tlsrpt". Unless a DKIM record explicitly
+			// specifies services (e.g. s=email), a record will work for TLS reports. The DKIM
+			// records seen used for TLS reporting in the wild don't explicitly set "s" for
+			// services.
+			// ../rfc/8460:326
+			if r.Status == dkim.StatusPass && matchesDomain(r.Sig.Domain) && r.Sig.Length < 0 && r.Record.ServiceAllowed("tlsrpt") {
 				ok = true
 				break
 			}
@@ -215,23 +255,27 @@ func analyze(ctx context.Context, log *mlog.Log, resolver dns.Resolver, d delive
 
 		if !ok {
 			log.Info("received mail to tlsrpt without acceptable DKIM signature, not processing as tls report")
-		} else if report, err := tlsrpt.ParseMessage(log, store.FileMsgReader(d.m.MsgPrefix, d.dataFile)); err != nil {
+			headers += "X-Mox-TLSReport-Error: no acceptable DKIM signature\r\n"
+		} else if reportJSON, err := tlsrpt.ParseMessage(log.Logger, store.FileMsgReader(d.m.MsgPrefix, d.dataFile)); err != nil {
 			log.Infox("parsing tls report", err)
+			headers += "X-Mox-TLSReport-Error: could not parse TLS report\r\n"
 		} else {
 			var known bool
-			for _, p := range report.Policies {
-				log.Info("tlsrpt policy domain", mlog.Field("domain", p.Policy.Domain))
+			for _, p := range reportJSON.Policies {
+				log.Info("tlsrpt policy domain", slog.String("domain", p.Policy.Domain))
 				if d, err := dns.ParseDomain(p.Policy.Domain); err != nil {
 					log.Infox("parsing domain in tls report", err)
-				} else if _, ok := mox.Conf.Domain(d); ok {
+				} else if _, ok := mox.Conf.Domain(d); ok || d == mox.Conf.Static.HostnameDomain {
 					known = true
 					break
 				}
 			}
 			if !known {
 				log.Info("tls report without one of configured domains, ignoring")
+				headers += "X-Mox-TLSReport-Error: report for unknown domain\r\n"
 			} else {
-				tlsReport = report
+				report := reportJSON.Convert()
+				tlsReport = &report
 			}
 		}
 	}
@@ -255,18 +299,21 @@ func analyze(ctx context.Context, log *mlog.Log, resolver dns.Resolver, d delive
 		})
 	})
 	if err != nil {
-		log.Infox("determining reputation", err, mlog.Field("message", d.m))
+		log.Infox("determining reputation", err, slog.Any("message", d.m))
 		return reject(smtp.C451LocalErr, smtp.SeSys3Other0, "error processing", err, reasonReputationError)
 	}
-	log.Info("reputation analyzed", mlog.Field("conclusive", conclusive), mlog.Field("isjunk", isjunk), mlog.Field("method", string(method)))
+	log.Info("reputation analyzed",
+		slog.Bool("conclusive", conclusive),
+		slog.Any("isjunk", isjunk),
+		slog.String("method", string(method)))
 	if conclusive {
 		if !*isjunk {
-			return analysis{accept: true, mailbox: mailbox, dmarcReport: dmarcReport, tlsReport: tlsReport, reason: reason}
+			return analysis{accept: true, mailbox: mailbox, dmarcReport: dmarcReport, tlsReport: tlsReport, reason: reason, dmarcOverrideReason: dmarcOverrideReason, headers: headers}
 		}
 		return reject(smtp.C451LocalErr, smtp.SeSys3Other0, "error processing", err, string(method))
 	} else if dmarcReport != nil || tlsReport != nil {
-		log.Info("accepting dmarc reporting or tlsrpt message without reputation")
-		return analysis{accept: true, mailbox: mailbox, dmarcReport: dmarcReport, tlsReport: tlsReport, reason: reasonReporting}
+		log.Info("accepting message with dmarc aggregate report or tls report without reputation")
+		return analysis{accept: true, mailbox: mailbox, dmarcReport: dmarcReport, tlsReport: tlsReport, reason: reasonReporting, dmarcOverrideReason: dmarcOverrideReason, headers: headers}
 	}
 	// If there was no previous message from sender or its domain, and we have an SPF
 	// (soft)fail, reject the message.
@@ -298,11 +345,11 @@ func analyze(ctx context.Context, log *mlog.Log, resolver dns.Resolver, d delive
 			log.Errorx("get key for verifying subject token", err)
 			return reject(smtp.C451LocalErr, smtp.SeSys3Other0, "error processing", err, reasonSubjectpassError)
 		}
-		err = subjectpass.Verify(log, d.dataFile, []byte(subjectpassKey), conf.SubjectPass.Period)
+		err = subjectpass.Verify(log.Logger, d.dataFile, []byte(subjectpassKey), conf.SubjectPass.Period)
 		pass := err == nil
-		log.Infox("pass by subject token", err, mlog.Field("pass", pass))
+		log.Infox("pass by subject token", err, slog.Bool("pass", pass))
 		if pass {
-			return analysis{accept: true, mailbox: mailbox, reason: reasonSubjectpass}
+			return analysis{accept: true, mailbox: mailbox, reason: reasonSubjectpass, dmarcOverrideReason: dmarcOverrideReason, headers: headers}
 		}
 	}
 
@@ -331,16 +378,50 @@ func analyze(ctx context.Context, log *mlog.Log, resolver dns.Resolver, d delive
 		jitter := (jitterRand.Float64() - 0.5) / 10
 		threshold := jf.Threshold + jitter
 
-		// With an iprev fail, we set a higher bar for content.
+		rcptToMatch := func(l []message.Address) bool {
+			// todo: we use Go's net/mail to parse message header addresses. it does not allow empty quoted strings (contrary to spec), leaving To empty. so we don't verify To address for that unusual case for now. ../rfc/5322:961 ../rfc/5322:743
+			if d.rcptAcc.rcptTo.Localpart == "" {
+				return true
+			}
+			for _, a := range l {
+				dom, err := dns.ParseDomain(a.Host)
+				if err != nil {
+					continue
+				}
+				lp, err := smtp.ParseLocalpart(a.User)
+				if err == nil && dom == d.rcptAcc.rcptTo.IPDomain.Domain && lp == d.rcptAcc.rcptTo.Localpart {
+					return true
+				}
+			}
+			return false
+		}
+
+		// todo: some of these checks should also apply for reputation-based analysis with a weak signal, e.g. verified dkim/spf signal from new domain.
+		// With an iprev fail, non-TLS connection or our address not in To/Cc header, we set a higher bar for content.
 		reason = reasonJunkContent
 		if suspiciousIPrevFail && threshold > 0.25 {
 			threshold = 0.25
-			log.Info("setting junk threshold due to iprev fail", mlog.Field("threshold", 0.25))
+			log.Info("setting junk threshold due to iprev fail", slog.Float64("threshold", threshold))
+			reason = reasonJunkContentStrict
+		} else if !d.tls && threshold > 0.25 {
+			threshold = 0.25
+			log.Info("setting junk threshold due to plaintext smtp", slog.Float64("threshold", threshold))
+			reason = reasonJunkContentStrict
+		} else if (rs == nil || !rs.IsForward) && threshold > 0.25 && !rcptToMatch(d.msgTo) && !rcptToMatch(d.msgCc) {
+			// A common theme in junk messages is your recipient address not being in the To/Cc
+			// headers. We may be in Bcc, but that's unusual for first-time senders. Some
+			// providers (e.g. gmail) does not DKIM-sign Bcc headers, so junk messages can be
+			// sent with matching Bcc headers. We don't get here for known senders.
+			threshold = 0.25
+			log.Info("setting junk threshold due to smtp rcpt to and message to/cc address mismatch", slog.Float64("threshold", threshold))
 			reason = reasonJunkContentStrict
 		}
 		accept = contentProb <= threshold
 		junkSubjectpass = contentProb < threshold-0.2
-		log.Info("content analyzed", mlog.Field("accept", accept), mlog.Field("contentprob", contentProb), mlog.Field("subjectpass", junkSubjectpass))
+		log.Info("content analyzed",
+			slog.Bool("accept", accept),
+			slog.Float64("contentprob", contentProb),
+			slog.Bool("subjectpass", junkSubjectpass))
 	} else if err != store.ErrNoJunkFilter {
 		log.Errorx("open junkfilter", err)
 		return reject(smtp.C451LocalErr, smtp.SeSys3Other0, "error processing", err, reasonJunkFilterError)
@@ -354,18 +435,18 @@ func analyze(ctx context.Context, log *mlog.Log, resolver dns.Resolver, d delive
 		blocked := func(zone dns.Domain) bool {
 			dnsblctx, dnsblcancel := context.WithTimeout(ctx, 30*time.Second)
 			defer dnsblcancel()
-			if !checkDNSBLHealth(dnsblctx, resolver, zone) {
-				log.Info("dnsbl not healthy, skipping", mlog.Field("zone", zone))
+			if !checkDNSBLHealth(dnsblctx, log, resolver, zone) {
+				log.Info("dnsbl not healthy, skipping", slog.Any("zone", zone))
 				return false
 			}
 
-			status, expl, err := dnsbl.Lookup(dnsblctx, resolver, zone, net.ParseIP(d.m.RemoteIP))
+			status, expl, err := dnsbl.Lookup(dnsblctx, log.Logger, resolver, zone, net.ParseIP(d.m.RemoteIP))
 			dnsblcancel()
 			if status == dnsbl.StatusFail {
-				log.Info("rejecting due to listing in dnsbl", mlog.Field("zone", zone), mlog.Field("explanation", expl))
+				log.Info("rejecting due to listing in dnsbl", slog.Any("zone", zone), slog.String("explanation", expl))
 				return true
 			} else if err != nil {
-				log.Infox("dnsbl lookup", err, mlog.Field("zone", zone), mlog.Field("status", status))
+				log.Infox("dnsbl lookup", err, slog.Any("zone", zone), slog.Any("status", status))
 			}
 			return false
 		}
@@ -382,12 +463,12 @@ func analyze(ctx context.Context, log *mlog.Log, resolver dns.Resolver, d delive
 	}
 
 	if accept {
-		return analysis{accept: true, mailbox: mailbox, reason: reasonNoBadSignals}
+		return analysis{accept: true, mailbox: mailbox, reason: reasonNoBadSignals, dmarcOverrideReason: dmarcOverrideReason, headers: headers}
 	}
 
 	if subjectpassKey != "" && d.dmarcResult.Status == dmarc.StatusPass && method == methodNone && (dnsblocklisted || junkSubjectpass) {
 		log.Info("permanent reject with subjectpass hint of moderately spammy email without reputation")
-		pass := subjectpass.Generate(d.msgFrom, []byte(subjectpassKey), time.Now())
+		pass := subjectpass.Generate(log.Logger, d.msgFrom, []byte(subjectpassKey), time.Now())
 		return reject(smtp.C550MailboxUnavail, smtp.SePol7DeliveryUnauth1, subjectpass.Explanation+pass, nil, reasonGiveSubjectpass)
 	}
 
