@@ -3,40 +3,44 @@ package smtpserver
 import (
 	"context"
 	"fmt"
-	"os"
+	"time"
 
 	"github.com/mjl-/mox/dsn"
+	"github.com/mjl-/mox/mlog"
+	"github.com/mjl-/mox/mox-"
 	"github.com/mjl-/mox/queue"
 	"github.com/mjl-/mox/smtp"
 	"github.com/mjl-/mox/store"
 )
 
 // compose dsn message and add it to the queue for delivery to rcptTo.
-func queueDSN(ctx context.Context, c *conn, rcptTo smtp.Path, m dsn.Message) error {
+func queueDSN(ctx context.Context, log mlog.Log, c *conn, rcptTo smtp.Path, m dsn.Message, requireTLS bool) error {
 	buf, err := m.Compose(c.log, false)
 	if err != nil {
 		return err
 	}
+	bufDKIM, err := mox.DKIMSign(ctx, c.log, m.From, false, buf)
+	log.Check(err, "dkim signing dsn")
+	buf = append([]byte(bufDKIM), buf...)
+
 	var bufUTF8 []byte
 	if c.smtputf8 {
 		bufUTF8, err = m.Compose(c.log, true)
 		if err != nil {
 			c.log.Errorx("composing dsn with utf-8 for incoming delivery for unknown user, continuing with ascii-only dsn", err)
+		} else {
+			bufUTF8DKIM, err := mox.DKIMSign(ctx, log, m.From, true, bufUTF8)
+			log.Check(err, "dkim signing dsn with utf8")
+			bufUTF8 = append([]byte(bufUTF8DKIM), bufUTF8...)
 		}
 	}
 
-	f, err := store.CreateMessageTemp("smtp-dsn")
+	f, err := store.CreateMessageTemp(c.log, "smtp-dsn")
 	if err != nil {
 		return fmt.Errorf("creating temp file: %w", err)
 	}
-	defer func() {
-		if f != nil {
-			err := os.Remove(f.Name())
-			c.log.Check(err, "removing temporary dsn message file")
-			err = f.Close()
-			c.log.Check(err, "closing temporary dsn message file")
-		}
-	}()
+	defer store.CloseRemoveTempFile(c.log, f, "smtpserver dsn message")
+
 	if _, err := f.Write([]byte(buf)); err != nil {
 		return fmt.Errorf("writing dsn file: %w", err)
 	}
@@ -46,11 +50,14 @@ func queueDSN(ctx context.Context, c *conn, rcptTo smtp.Path, m dsn.Message) err
 	// ../rfc/3464:433
 	const has8bit = false
 	const smtputf8 = false
-	if _, err := queue.Add(ctx, c.log, "", smtp.Path{}, rcptTo, has8bit, smtputf8, int64(len(buf)), m.MessageID, nil, f, bufUTF8, true); err != nil {
+	var reqTLS *bool
+	if requireTLS {
+		reqTLS = &requireTLS
+	}
+	qm := queue.MakeMsg(smtp.Path{}, rcptTo, has8bit, smtputf8, int64(len(buf)), m.MessageID, nil, reqTLS, time.Now())
+	qm.DSNUTF8 = bufUTF8
+	if err := queue.Add(ctx, c.log, "", f, qm); err != nil {
 		return err
 	}
-	err = f.Close()
-	c.log.Check(err, "closing dsn file")
-	f = nil
 	return nil
 }
