@@ -10,9 +10,12 @@ import (
 	"time"
 
 	"github.com/mjl-/mox/dns"
+	"github.com/mjl-/mox/mlog"
 	"github.com/mjl-/mox/smtp"
 )
 
+var pkglog = mlog.New("spf", nil)
+
 func TestLookup(t *testing.T) {
 	resolver := dns.MockResolver{
 		TXT: map[string][]string{
@@ -22,8 +25,8 @@ func TestLookup(t *testing.T) {
 			"nonspf.example.":    {"something else"},
 			"ok.example.":        {"v=spf1"},
 		},
-		Fail: map[dns.Mockreq]struct{}{
-			{Type: "txt", Name: "temperror.example."}: {},
+		Fail: []string{
+			"txt temperror.example.",
 		},
 	}
 
@@ -31,7 +34,7 @@ func TestLookup(t *testing.T) {
 		t.Helper()
 
 		d := dns.Domain{ASCII: domain}
-		status, txt, record, err := Lookup(context.Background(), resolver, d)
+		status, txt, record, _, err := Lookup(context.Background(), pkglog.Logger, resolver, d)
 		if (err == nil) != (expErr == nil) || err != nil && !errors.Is(err, expErr) {
 			t.Fatalf("got err %v, expected err %v", err, expErr)
 		}
@@ -99,7 +102,7 @@ func TestExpand(t *testing.T) {
 			args.RemoteIP = mustParseIP(ip)
 		}
 
-		r, err := expandDomainSpec(ctx, resolver, macro, args, dns)
+		r, _, err := expandDomainSpec(ctx, resolver, macro, args, dns)
 		if (err == nil) != (exp != "") {
 			t.Fatalf("got err %v, expected expansion %q, for macro %q", err, exp, macro)
 		}
@@ -245,7 +248,6 @@ func TestVerify(t *testing.T) {
 				{Host: "mail-c.example.org.", Pref: 10},
 			},
 		},
-		Fail: map[dns.Mockreq]struct{}{},
 	}
 
 	ctx := context.Background()
@@ -260,7 +262,7 @@ func TestVerify(t *testing.T) {
 			LocalIP:           xip("127.0.0.1"),
 			LocalHostname:     dns.Domain{ASCII: "localhost"},
 		}
-		received, _, _, err := Verify(ctx, r, args)
+		received, _, _, _, err := Verify(ctx, pkglog.Logger, r, args)
 		if received.Result != status {
 			t.Fatalf("got status %q, expected %q, for ip %q (err %v)", received.Result, status, ip, err)
 		}
@@ -346,7 +348,7 @@ func TestVerifyMultipleDomain(t *testing.T) {
 			LocalIP:        net.ParseIP("127.0.0.1"),
 			LocalHostname:  dns.Domain{ASCII: "localhost"},
 		}
-		received, _, _, err := Verify(context.Background(), resolver, args)
+		received, _, _, _, err := Verify(context.Background(), pkglog.Logger, resolver, args)
 		if err != nil {
 			t.Fatalf("unexpected error: %s", err)
 		}
@@ -371,7 +373,7 @@ func TestVerifyScenarios(t *testing.T) {
 	test := func(resolver dns.Resolver, args Args, expStatus Status, expDomain string, expExpl string, expErr error) {
 		t.Helper()
 
-		recv, d, expl, err := Verify(context.Background(), resolver, args)
+		recv, d, expl, _, err := Verify(context.Background(), pkglog.Logger, resolver, args)
 		if (err == nil) != (expErr == nil) || err != nil && !errors.Is(err, expErr) {
 			t.Fatalf("got err %v, expected %v", err, expErr)
 		}
@@ -441,8 +443,8 @@ func TestVerifyScenarios(t *testing.T) {
 			"2001:db8::1": {"mail.mox.example."},
 			"10.0.1.1":    {"mx1.many-mx.example.", "mx2.many-mx.example.", "mx3.many-mx.example.", "mx4.many-mx.example.", "mx5.many-mx.example.", "mx6.many-mx.example.", "mx7.many-mx.example.", "mx8.many-mx.example.", "mx9.many-mx.example.", "mx10.many-mx.example.", "mx11.many-mx.example."},
 		},
-		Fail: map[dns.Mockreq]struct{}{
-			{Type: "txt", Name: "temperror.example."}: {},
+		Fail: []string{
+			"txt temperror.example.",
 		},
 	}
 
@@ -506,7 +508,7 @@ func TestEvaluate(t *testing.T) {
 	record := &Record{}
 	resolver := dns.MockResolver{}
 	args := Args{}
-	status, _, _, _ := Evaluate(context.Background(), record, resolver, args)
+	status, _, _, _, _ := Evaluate(context.Background(), pkglog.Logger, record, resolver, args)
 	if status != StatusNone {
 		t.Fatalf("got status %q, expected none", status)
 	}
@@ -514,7 +516,7 @@ func TestEvaluate(t *testing.T) {
 	args = Args{
 		HelloDomain: dns.IPDomain{Domain: dns.Domain{ASCII: "test.example"}},
 	}
-	status, mechanism, _, err := Evaluate(context.Background(), record, resolver, args)
+	status, mechanism, _, _, err := Evaluate(context.Background(), pkglog.Logger, record, resolver, args)
 	if status != StatusNeutral || mechanism != "default" || err != nil {
 		t.Fatalf("got status %q, mechanism %q, err %v, expected neutral, default, no error", status, mechanism, err)
 	}
