@@ -4,19 +4,24 @@ import (
 	"context"
 	"fmt"
 	"net/http"
+	"net/http/httptest"
 	"os"
+	"path/filepath"
 	"runtime/debug"
+	"slices"
 	"testing"
 
 	"github.com/mjl-/bstore"
 	"github.com/mjl-/sherpa"
 
+	"github.com/mjl-/mox/dns"
+	"github.com/mjl-/mox/mlog"
 	"github.com/mjl-/mox/mox-"
 	"github.com/mjl-/mox/queue"
 	"github.com/mjl-/mox/store"
 )
 
-func tneedError(t *testing.T, fn func()) {
+func tneedErrorCode(t *testing.T, code string, fn func()) {
 	t.Helper()
 	defer func() {
 		t.Helper()
@@ -27,33 +32,40 @@ func tneedError(t *testing.T, fn func()) {
 		}
 		if err, ok := x.(*sherpa.Error); !ok {
 			debug.PrintStack()
-			t.Fatalf("expected sherpa user error, saw %#v", x)
-		} else if err.Code != "user:error" {
+			t.Fatalf("expected sherpa error, saw %#v", x)
+		} else if err.Code != code {
 			debug.PrintStack()
-			t.Fatalf("expected sherpa user error, saw other sherpa error %#v", err)
+			t.Fatalf("expected sherpa error code %q, saw other sherpa error %#v", code, err)
 		}
 	}()
 
 	fn()
 }
 
+func tneedError(t *testing.T, fn func()) {
+	tneedErrorCode(t, "user:error", fn)
+}
+
 // Test API calls.
 // todo: test that the actions make the changes they claim to make. we currently just call the functions and have only limited checks that state changed.
 func TestAPI(t *testing.T) {
 	mox.LimitersInit()
 	os.RemoveAll("../testdata/webmail/data")
 	mox.Context = ctxbg
-	mox.ConfigStaticPath = "../testdata/webmail/mox.conf"
+	mox.ConfigStaticPath = filepath.FromSlash("../testdata/webmail/mox.conf")
 	mox.MustLoadConfig(true, false)
 	defer store.Switchboard()()
 
-	acc, err := store.OpenAccount("mjl")
+	log := mlog.New("webmail", nil)
+	acc, err := store.OpenAccount(log, "mjl")
 	tcheck(t, err, "open account")
-	err = acc.SetPassword("test1234")
+	const pw0 = "te\u0301st \u00a0\u2002\u200a" // NFD and various unicode spaces.
+	const pw1 = "tést    "                      // PRECIS normalized, with NFC.
+	err = acc.SetPassword(log, pw0)
 	tcheck(t, err, "set password")
 	defer func() {
 		err := acc.Close()
-		xlog.Check(err, "closing account")
+		pkglog.Check(err, "closing account")
 	}()
 
 	var zerom store.Message
@@ -73,8 +85,58 @@ func TestAPI(t *testing.T) {
 		tdeliver(t, acc, tm)
 	}
 
-	api := Webmail{maxMessageSize: 1024 * 1024}
-	reqInfo := requestInfo{"mjl@mox.example", "mjl", &http.Request{}}
+	api := Webmail{maxMessageSize: 1024 * 1024, cookiePath: "/webmail/"}
+
+	// Test login, and rate limiter.
+	loginReqInfo := requestInfo{"mjl@mox.example", "mjl", "", httptest.NewRecorder(), &http.Request{RemoteAddr: "1.1.1.1:1234"}}
+	loginctx := context.WithValue(ctxbg, requestInfoCtxKey, loginReqInfo)
+
+	// Missing login token.
+	tneedErrorCode(t, "user:error", func() { api.Login(loginctx, "", "mjl@mox.example", pw0) })
+
+	// Login with loginToken.
+	loginCookie := &http.Cookie{Name: "webmaillogin"}
+	loginCookie.Value = api.LoginPrep(loginctx)
+	loginReqInfo.Request.Header = http.Header{"Cookie": []string{loginCookie.String()}}
+
+	testLogin := func(username, password string, expErrCodes ...string) {
+		t.Helper()
+
+		defer func() {
+			x := recover()
+			expErr := len(expErrCodes) > 0
+			if (x != nil) != expErr {
+				t.Fatalf("got %v, expected codes %v, for username %q, password %q", x, expErrCodes, username, password)
+			}
+			if x == nil {
+				return
+			} else if err, ok := x.(*sherpa.Error); !ok {
+				t.Fatalf("got %#v, expected at most *sherpa.Error", x)
+			} else if !slices.Contains(expErrCodes, err.Code) {
+				t.Fatalf("got error code %q, expected %v", err.Code, expErrCodes)
+			}
+		}()
+
+		api.Login(loginctx, loginCookie.Value, username, password)
+	}
+	testLogin("mjl@mox.example", pw0)
+	testLogin("mjl@mox.example", pw1)
+	testLogin("móx@mox.example", pw1)       // NFC username
+	testLogin("mo\u0301x@mox.example", pw1) // NFD username
+	testLogin("mjl@mox.example", pw1+" ", "user:loginFailed")
+	testLogin("nouser@mox.example", pw0, "user:loginFailed")
+	testLogin("nouser@bad.example", pw0, "user:loginFailed")
+	for i := 3; i < 10; i++ {
+		testLogin("bad@bad.example", pw0, "user:loginFailed")
+	}
+	// Ensure rate limiter is triggered, also for slow tests.
+	for i := 0; i < 10; i++ {
+		testLogin("bad@bad.example", pw0, "user:loginFailed", "user:error")
+	}
+	testLogin("bad@bad.example", pw0, "user:error")
+
+	// Context with different IP, for clear rate limit history.
+	reqInfo := requestInfo{"mjl@mox.example", "mjl", "", nil, &http.Request{RemoteAddr: "127.0.0.1:1234"}}
 	ctx := context.WithValue(ctxbg, requestInfoCtxKey, reqInfo)
 
 	// FlagsAdd
@@ -361,4 +423,15 @@ func TestAPI(t *testing.T) {
 	l, full = api.CompleteRecipient(ctx, "cc2")
 	tcompare(t, l, []string{"mjl cc2 <mjl+cc2@mox.example>"})
 	tcompare(t, full, true)
+
+	// RecipientSecurity
+	resolver := dns.MockResolver{}
+	rs, err := recipientSecurity(ctx, resolver, "mjl@a.mox.example")
+	tcompare(t, err, nil)
+	tcompare(t, rs, RecipientSecurity{SecurityResultUnknown, SecurityResultNo, SecurityResultNo, SecurityResultNo, SecurityResultUnknown})
+	err = acc.DB.Insert(ctx, &store.RecipientDomainTLS{Domain: "a.mox.example", STARTTLS: true, RequireTLS: false})
+	tcheck(t, err, "insert recipient domain tls info")
+	rs, err = recipientSecurity(ctx, resolver, "mjl@a.mox.example")
+	tcompare(t, err, nil)
+	tcompare(t, rs, RecipientSecurity{SecurityResultYes, SecurityResultNo, SecurityResultNo, SecurityResultNo, SecurityResultNo})
 }
