@@ -4,13 +4,17 @@ import (
 	"bufio"
 	"fmt"
 	"io"
+	"log/slog"
 	"mime"
 	"net/url"
 	"strings"
 
+	"golang.org/x/text/encoding/ianaindex"
+
 	"github.com/mjl-/mox/dns"
 	"github.com/mjl-/mox/message"
 	"github.com/mjl-/mox/mlog"
+	"github.com/mjl-/mox/mox-"
 	"github.com/mjl-/mox/moxio"
 	"github.com/mjl-/mox/smtp"
 	"github.com/mjl-/mox/store"
@@ -18,7 +22,56 @@ import (
 
 // todo: we should have all needed information for messageItem in store.Message (perhaps some data in message.Part) for fast access, not having to parse the on-disk message file.
 
-func messageItem(log *mlog.Log, m store.Message, state *msgState) (MessageItem, error) {
+var wordDecoder = mime.WordDecoder{
+	CharsetReader: func(charset string, r io.Reader) (io.Reader, error) {
+		switch strings.ToLower(charset) {
+		case "", "us-ascii", "utf-8":
+			return r, nil
+		}
+		enc, _ := ianaindex.MIME.Encoding(charset)
+		if enc == nil {
+			enc, _ = ianaindex.IANA.Encoding(charset)
+		}
+		if enc == nil {
+			return r, fmt.Errorf("unknown charset %q", charset)
+		}
+		return enc.NewDecoder().Reader(r), nil
+	},
+}
+
+// Attempt q/b-word-decode name, coming from Content-Type "name" field or
+// Content-Disposition "filename" field.
+//
+// RFC 2231 specify an encoding for non-ascii values in mime header parameters. But
+// it appears common practice to instead just q/b-word encode the values.
+// Thunderbird and gmail.com do this for the Content-Type "name" parameter.
+// gmail.com also does that for the Content-Disposition "filename" parameter, where
+// Thunderbird uses the RFC 2231-defined encoding. Go's mime.ParseMediaType parses
+// the mechanism specified in RFC 2231 only. The value for "name" we get here would
+// already be decoded properly for standards-compliant headers, like
+// "filename*0*=UTF-8”%...; filename*1*=%.... We'll look for Q/B-word encoding
+// markers ("=?"-prefix or "?="-suffix) and try to decode if present. This would
+// only cause trouble for filenames having this prefix/suffix.
+func tryDecodeParam(log mlog.Log, name string) string {
+	if name == "" || !strings.HasPrefix(name, "=?") && !strings.HasSuffix(name, "?=") {
+		return name
+	}
+	// todo: find where this is allowed. it seems quite common. perhaps we should remove the pedantic check?
+	if mox.Pedantic {
+		log.Debug("attachment contains rfc2047 q/b-word-encoded mime parameter instead of rfc2231-encoded", slog.String("name", name))
+		return name
+	}
+	s, err := wordDecoder.DecodeHeader(name)
+	if err != nil {
+		log.Debugx("q/b-word decoding mime parameter", err, slog.String("name", name))
+		return name
+	}
+	return s
+}
+
+// todo: mime.FormatMediaType does not wrap long lines. should do it ourselves, and split header into several parts (if commonly supported).
+
+func messageItem(log mlog.Log, m store.Message, state *msgState) (MessageItem, error) {
 	pm, err := parsedMessage(log, m, state, false, true)
 	if err != nil {
 		return MessageItem{}, fmt.Errorf("parsing message %d for item: %v", m.ID, err)
@@ -127,7 +180,7 @@ func formatFirstLine(r io.Reader) (string, error) {
 	return result, scanner.Err()
 }
 
-func parsedMessage(log *mlog.Log, m store.Message, state *msgState, full, msgitem bool) (pm ParsedMessage, rerr error) {
+func parsedMessage(log mlog.Log, m store.Message, state *msgState, full, msgitem bool) (pm ParsedMessage, rerr error) {
 	if full || msgitem {
 		if !state.ensurePart(m, true) {
 			return pm, state.err
@@ -206,16 +259,15 @@ func parsedMessage(log *mlog.Log, m store.Message, state *msgState, full, msgite
 			if full || msgitem {
 				// todo: should have this, and perhaps all content-* headers, preparsed in message.Part?
 				h, err := p.Header()
-				log.Check(err, "parsing attachment headers", mlog.Field("msgid", m.ID))
+				log.Check(err, "parsing attachment headers", slog.Int64("msgid", m.ID))
 				cp := h.Get("Content-Disposition")
 				if cp != "" {
 					disp, params, err := mime.ParseMediaType(cp)
-					log.Check(err, "parsing content-disposition", mlog.Field("cp", cp))
+					log.Check(err, "parsing content-disposition", slog.String("cp", cp))
 					if strings.EqualFold(disp, "attachment") {
-						// todo: should we be decoding these names? i've seen messages with regular q-word style mime-encoding, not the one specified in ../rfc/2231:210
-						name := p.ContentTypeParams["name"]
+						name := tryDecodeParam(log, p.ContentTypeParams["name"])
 						if name == "" {
-							name = params["filename"]
+							name = tryDecodeParam(log, params["filename"])
 						}
 						pm.attachments = append(pm.attachments, Attachment{path, name, p})
 						return
@@ -285,16 +337,16 @@ func parsedMessage(log *mlog.Log, m store.Message, state *msgState, full, msgite
 					return
 				}
 
-				name, ok := p.ContentTypeParams["name"]
-				if !ok && (full || msgitem) {
+				name := tryDecodeParam(log, p.ContentTypeParams["name"])
+				if name == "" && (full || msgitem) {
 					// todo: should have this, and perhaps all content-* headers, preparsed in message.Part?
 					h, err := p.Header()
-					log.Check(err, "parsing attachment headers", mlog.Field("msgid", m.ID))
+					log.Check(err, "parsing attachment headers", slog.Int64("msgid", m.ID))
 					cp := h.Get("Content-Disposition")
 					if cp != "" {
 						_, params, err := mime.ParseMediaType(cp)
-						log.Check(err, "parsing content-disposition", mlog.Field("cp", cp))
-						name = params["filename"]
+						log.Check(err, "parsing content-disposition", slog.String("cp", cp))
+						name = tryDecodeParam(log, params["filename"])
 					}
 				}
 				pm.attachments = append(pm.attachments, Attachment{path, name, p})
